@@ -1,5 +1,6 @@
 import enum
 import platform
+import os
 
 import copy
 from aiosmb.commons.connection.credential import *
@@ -12,8 +13,6 @@ from minikerberos.common.proxy import KerberosProxy
 from minikerberos.common.creds import KerberosCredential
 from minikerberos.common.spn import KerberosSPN
 
-from minikerberos.network.selector import KerberosClientSocketSelector
-
 
 if platform.system().upper() == 'WINDOWS':
 	from aiosmb.authentication.kerberos.sspi import SMBKerberosSSPI
@@ -62,26 +61,43 @@ class AuthenticatorBuilder:
 			if target.dc_ip is None:
 				raise Exception('target must have a dc_ip for kerberos!')
 			
-			kc = KerberosCredential()
-			kc.username = creds.username
-			kc.domain = creds.domain
-			if creds.secret_type == SMBCredentialsSecretType.PASSWORD:
-				kc.password = creds.secret
-			elif creds.secret_type == SMBCredentialsSecretType.NT:
-				kc.nt_hash = creds.secret
-				
-			elif creds.secret_type == SMBCredentialsSecretType.AES:
-				if len(creds.secret) == 32:
-					kc.kerberos_key_aes_128 = creds.secret
-				elif len(creds.secret) == 64:
-					kc.kerberos_key_aes_256 = creds.secret
-					
-			elif creds.secret_type == SMBCredentialsSecretType.RC4:
-				kc.kerberos_key_rc4 = creds.secret
+			if creds.secret_type == SMBCredentialsSecretType.KEYTAB:
+				filename = creds.secret
+				if creds.secret.upper() == 'ENV':
+					filename = os.environ['KRB5KEYTAB']
+
+				kc = KerberosCredential.from_keytab(filename, creds.username, creds.domain)
+
+			elif creds.secret_type == SMBCredentialsSecretType.CCACHE:
+				filename = creds.secret
+				if creds.secret.upper() == 'ENV':
+					try:
+						filename = os.environ['KRB5CCACHE']
+					except:
+						raise Exception('Kerberos auth missing environment variable KRB5CCACHE')
+				kc = KerberosCredential.from_ccache_file(filename)
+				kc.username = creds.username
+				kc.domain = creds.domain
 			
-			elif creds.secret_type == SMBCredentialsSecretType.RC4:
-				kc.ccache = creds.secret
 			else:
+				kc = KerberosCredential()
+				kc.username = creds.username
+				kc.domain = creds.domain
+				if creds.secret_type == SMBCredentialsSecretType.PASSWORD:
+					kc.password = creds.secret
+				elif creds.secret_type == SMBCredentialsSecretType.NT:
+					kc.nt_hash = creds.secret
+					
+				elif creds.secret_type == SMBCredentialsSecretType.AES:
+					if len(creds.secret) == 32:
+						kc.kerberos_key_aes_128 = creds.secret
+					elif len(creds.secret) == 64:
+						kc.kerberos_key_aes_256 = creds.secret
+						
+				elif creds.secret_type == SMBCredentialsSecretType.RC4:
+					kc.kerberos_key_rc4 = creds.secret
+
+			if kc is None:
 				raise Exception('No suitable secret type found to set up kerberos!')
 			
 				
@@ -136,6 +152,44 @@ class AuthenticatorBuilder:
 			spneg.add_auth_context('NTLMSSP - Microsoft NTLM Security Support Provider', handler)
 			return spneg
 
+		elif creds.authentication_type.value.startswith('MPN'):
+			if creds.authentication_type in [SMBAuthProtocol.MPN_SSL_NTLM, SMBAuthProtocol.MPN_NTLM]:
+				from aiosmb.authentication.ntlm.mpn import SMBNTLMMPN
+				ntlmcred = SMBMPNCredential()
+				ntlmcred.type = 'NTLM'
+				ntlmcred.is_ssl = True if creds.authentication_type == SMBAuthProtocol.MPN_SSL_NTLM else False
+				ntlmcred.parse_settings(creds.settings)
+				
+				handler = SMBNTLMMPN(ntlmcred)
+				#setting up SPNEGO
+				spneg = SPNEGO()
+				spneg.add_auth_context('NTLMSSP - Microsoft NTLM Security Support Provider', handler)
+				return spneg
+
+			elif creds.authentication_type in [SMBAuthProtocol.MPN_SSL_KERBEROS, SMBAuthProtocol.MPN_KERBEROS]:
+				from aiosmb.authentication.kerberos.mpn import SMBKerberosMPN
+
+				ntlmcred = SMBMPNCredential()
+				ntlmcred.type = 'KERBEROS'
+				ntlmcred.target = creds.target
+				if creds.username is not None:
+					ntlmcred.username = '<CURRENT>'
+				if creds.domain is not None:
+					ntlmcred.domain = '<CURRENT>'
+				if creds.secret is not None:
+					ntlmcred.password = '<CURRENT>'
+				ntlmcred.is_guest = False
+				ntlmcred.is_ssl = True if creds.authentication_type == SMBAuthProtocol.MPN_SSL_KERBEROS else False
+				ntlmcred.parse_settings(creds.settings)
+
+				handler = SMBKerberosMPN(ntlmcred)
+				#setting up SPNEGO
+				spneg = SPNEGO()
+				spneg.add_auth_context('MS KRB5 - Microsoft Kerberos 5', handler)
+				return spneg
+
+				
+
 		elif creds.authentication_type.value.startswith('MULTIPLEXOR'):
 			if creds.authentication_type in [SMBAuthProtocol.MULTIPLEXOR_SSL_NTLM, SMBAuthProtocol.MULTIPLEXOR_NTLM]:
 				from aiosmb.authentication.ntlm.multiplexor import SMBNTLMMultiplexor
