@@ -32,7 +32,7 @@ def req_traceback(funct):
 	return wrapper
 
 class SMBClient(aiocmd.PromptToolkitCmd):
-	def __init__(self, url = None):
+	def __init__(self, url = None, silent = False):
 		aiocmd.PromptToolkitCmd.__init__(self, ignore_sigint=False) #Setting this to false, since True doesnt work on windows...
 		self.conn_url = None
 		if url is not None:
@@ -40,6 +40,7 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 		self.connection = None
 		self.machine = None
 		self.is_anon = False
+		self.silent = silent
 
 		self.shares = {} #name -> share
 		self.__current_share = None
@@ -67,7 +68,8 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			if err is not None:
 				raise err
 			self.machine = SMBMachine(self.connection)
-			print('Login success')
+			if self.silent is False:
+				print('Login success')
 			return True, None
 		except Exception as e:
 			traceback.print_exc()
@@ -92,6 +94,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 	async def do_shares(self, show = True):
 		"""Lists available shares"""
 		try:
+			if self.machine is None:
+				print('Not logged in! Use "login" first!')
+				return False, Exception('Not logged in!')
 			async for share, err in self.machine.list_shares():
 				if err is not None:
 					raise err
@@ -256,7 +261,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 				raise err
 			self.__current_directory = self.__current_share.subdirs[''] #this is the entry directory
 			self.prompt = '[%s]$ ' % self.__current_directory.unc_path
-			await self.do_ls(False)
+			_, err = await self.do_refreshcurdir()
+			if err is not None:
+				raise err
 			return True, None
 
 		except SMBException as e:
@@ -275,8 +282,10 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 			
-	
-	async def do_ls(self, show = True):
+	async def do_dir(self):
+		return await self.do_ls()
+
+	async def do_ls(self):
 		try:
 			if self.__current_share is None:
 				print('No share selected!')
@@ -285,11 +294,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 				print('No directory selected!')
 				return
 			
-			#print(self.__current_directory)
-			async for entry in self.machine.list_directory(self.__current_directory):
-				if show == True:
-					print(entry)
-
+			for entry in self.__current_directory.get_console_output():
+				print(entry)
+			
 			return True, None
 		except SMBException as e:
 			logger.debug(traceback.format_exc())
@@ -307,6 +314,17 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 
+	async def do_refreshcurdir(self):
+		try:
+			async for entry in self.machine.list_directory(self.__current_directory):
+				#no need to put here anything, the dir bject will store the refreshed data
+				a = 1
+			
+			return True, None
+		except Exception as e:
+			traceback.print_exc()
+			return None, e
+
 	async def do_cd(self, directory_name):
 		try:
 			if self.__current_share is None:
@@ -328,8 +346,10 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			else:
 				self.__current_directory = self.__current_directory.subdirs[directory_name]
 				self.prompt = '[%s] $' % (self.__current_directory.unc_path)
-				await self.do_ls(False)
-			
+				_, err = await self.do_refreshcurdir()
+				if err is not None:
+					raise err
+
 				return True, None
 			
 		except SMBException as e:
@@ -358,16 +378,16 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			return []
 		return list(self.__current_directory.files.keys())
 
-	async def do_sid(self, file_name):
+	async def do_getfilesd(self, file_name):
 		try:
 			if file_name not in self.__current_directory.files:
 				print('file not in current directory!')
 				return False, None
 			file_obj = self.__current_directory.files[file_name]
-			sid, err = await file_obj.get_security_descriptor(self.connection)
+			sd, err = await file_obj.get_security_descriptor(self.connection)
 			if err is not None:
 				raise err
-			print(str(sid))
+			print(sd.to_sddl())
 			return True, None
 
 		except SMBException as e:
@@ -386,12 +406,12 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			traceback.print_exc()
 			return None, e
 
-	async def do_dirsid(self):
+	async def do_getdirsd(self):
 		try:
-			sid, err = await self.__current_directory.get_security_descriptor(self.connection)
+			sd, err = await self.__current_directory.get_security_descriptor(self.connection)
 			if err is not None:
 				raise err
-			print(str(sid))
+			print(str(sd.to_sddl()))
 			return True, None
 		except SMBException as e:
 			logger.debug(traceback.format_exc())
@@ -537,9 +557,13 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			dst = '\\%s\\%s\\%s' % (self.__current_share.name, self.__current_directory.fullpath , basename)
 			_, err = await self.machine.put_file(file_name, dst)
 			if err is not None:
-				raise err
+				print('Failed to put file! Reason: %s' % err)
+				return False, err
 			print('File uploaded!')
-			await self.do_ls(False)
+			_, err = await self.do_refreshcurdir()
+			if err is not None:
+				raise err
+			
 			return True, None
 
 		except SMBException as e:
@@ -567,7 +591,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			if err is not None:
 				raise err
 			print('File deleted!')
-			await self.do_ls(False)
+			_, err = await self.do_refreshcurdir()
+			if err is not None:
+				raise err
 			return True, None
 
 		except SMBException as e:
@@ -663,7 +689,9 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			if err is not None:
 				raise err
 			print('Directory created!')
-			await self.do_ls(False)
+			_, err = await self.do_refreshcurdir()
+			if err is not None:
+				raise err
 			return True, None
 
 		except SMBException as e:
@@ -915,7 +943,7 @@ class SMBClient(aiocmd.PromptToolkitCmd):
 			return None, e
 
 async def amain(args):
-	client = SMBClient(args.smb_url)
+	client = SMBClient(args.smb_url, silent = args.silent)
 	if len(args.commands) == 0:
 		if args.no_interactive is True:
 			print('Not starting interactive!')
@@ -941,12 +969,14 @@ def main():
 	
 	parser = argparse.ArgumentParser(description='Interactive SMB client')
 	parser.add_argument('-v', '--verbose', action='count', default=0)
+	parser.add_argument('-s', '--silent', action='store_true', help='do not print banner')
 	parser.add_argument('-n', '--no-interactive', action='store_true')
 	parser.add_argument('smb_url', help = 'Connection string that describes the authentication and target. Example: smb+ntlm-password://TEST\\Administrator:password@10.10.10.2')
 	parser.add_argument('commands', nargs='*')
 	
 	args = parser.parse_args()
-	print(__banner__)
+	if args.silent is False:
+		print(__banner__)
 
 	if args.verbose >=1:
 		logger.setLevel(logging.DEBUG)
